@@ -367,7 +367,7 @@ function _select_package_version(package, requireinfo, locked_requireinfo)
         if require_version and #require_version == 40 and require_version:match("%w+") then
             version, source = require_version, "commit"
         else
-            version, source = require_version ~= "latest" and require_version or "master", "branch"
+            version, source = require_version ~= "latest" and require_version or "@default", "branch"
         end
     end
     -- local source package? we use a phony version
@@ -706,6 +706,17 @@ end
 -- load required packages
 function _load_package(packagename, requireinfo, opt)
 
+    -- check circular dependency
+    opt = opt or {}
+    if opt.requirepath then
+        local splitinfo = opt.requirepath:split(".", {plain = true})
+        if #splitinfo > 3 and
+            splitinfo[1] == splitinfo[#splitinfo - 1] and
+            splitinfo[2] == splitinfo[#splitinfo] then
+            raise("circular dependency(%s) detected in package(%s)!", opt.requirepath, splitinfo[1])
+        end
+    end
+
     -- strip trailng ~tag, e.g. zlib~debug
     local displayname
     if packagename:find('~', 1, true) then
